//
// INA219_library.cpp
// Class library C++ code
// ----------------------------------
// Developed with embedXcode
// http://embedXcode.weebly.com
//
// Project INA219_main
//
// Created by Rei VILO, Jun 06, 2013
// embedXcode.weebly.com
//
//
// Copyright © Rei VILO, 2013
// Licence CC = BY NC SA
//
// See INA219_library.h and ReadMe.txt for references
//


// Library header
#include "INA219_library.h"


const uint8_t INA219_CONFIGURATION = 0x00; // configuration register
const uint8_t INA219_SHUNT_VOLTAGE = 0x01; // shunt voltage register
const uint8_t INA219_BUS_VOLTAGE   = 0x02; // bus voltage register
const uint8_t INA219_POWER         = 0x03; // power register
const uint8_t INA219_CURRENT       = 0x04; // current register
const uint8_t INA219_CALIBRATION   = 0x05; // calibration register
const uint8_t INA219_I2C_ADDRESS   = 0x40; // I2C address


// Code
INA219::INA219() {
    ;
}

void INA219::begin()
{
    // default = bus 32 V, shunt 320 mV, 12-bit, continuous
    _writeRegister(INA219_CONFIGURATION, 0b0011100110011111);
    // same but with (1 << 6) = average of 8
    //_writeRegister(INA219_CONFIGURATION, 0b0011100111011111);
    _writeRegister(INA219_CALIBRATION,   0x1000);
}

String INA219::WhoAmI()
{
    return "INA219";
}

void INA219::get()
{ 
    _bus_mV     = _readRegister(INA219_BUS_VOLTAGE);
    _overflow   = bitRead(_bus_mV, 0);                  // overflow bit
    _ready      = bitRead(_bus_mV, 1);                  // ready bit
    _bus_mV     = (_bus_mV >> 3) * 4;                   // 4 mV/LSB

    _current_mA = _readRegister(INA219_CURRENT);
    if bitRead(_current_mA, 15) _current_mA = 0;        // negative
    _current_mA = _current_mA / 10;                     // 0.1 mA/LSB

    _shunt_mV   = _readRegister(INA219_SHUNT_VOLTAGE);
    _power_mW   = _readRegister(INA219_POWER) * 2;      // 2 mW/LSB
}

uint32_t INA219::mV()
{
    return _bus_mV;
}

uint32_t INA219::mA()
{
    return _current_mA;
}

uint32_t INA219::mW()
{
    return _power_mW;
}

bool INA219::overflow()
{
    return _overflow;
}

bool INA219::ready()
{
    return _ready;
}


// Utilities
void INA219::_writeRegister(uint8_t reg, uint16_t data)
{
    Wire.beginTransmission(INA219_I2C_ADDRESS);
    Wire.write(reg);
    Wire.write(highByte(data));
    Wire.write(lowByte(data));
    Wire.endTransmission();
}

uint16_t INA219::_readRegister(uint8_t reg)
{
    Wire.beginTransmission(INA219_I2C_ADDRESS);
    Wire.write(reg);
    Wire.endTransmission();
    
    Wire.requestFrom(0x40, 2);
    return (uint16_t)(Wire.read() << 8) + Wire.read();
}




